__author__='sigilis.com'
__version__='0.1'
from direct.actor.Actor import Actor
from direct.showbase.DirectObject import DirectObject
from pandac.PandaModules import Vec3, GeomNode
from direct.task.Task import Task
from direct.gui.OnscreenText import OnscreenText
from pandac.PandaModules import PandaNode,NodePath,Camera,TextNode    
from pandac.PandaModules import CollisionTraverser,CollisionNode
from pandac.PandaModules import CollisionHandlerQueue,CollisionRay,BitMask32
from pandac.PandaModules import CollisionSphere,CollisionSegment

import math

VIZRAD=100000

class OAvatar(Actor,DirectObject):
	def __init__(self,user=None,terrain=None,
			sx=None,sy=None,sz=None,eyeheight=None):
		self.user=user		
		models="models/%s" % self.user.modelType		
		anims= {"run":"models/%s-run" % self.user.modelType,
			"walk":"models/%s-walk" % self.user.modelType
			}
		
		Actor.__init__(self,models,anims)
		self.isMoving=False
		self.headchange=True
		
		self.eyeheight=eyeheight
		if sx!=None and sy!=None:
			self.reparentTo(render)
			self.terrain=terrain
			if sz==None:
				StartPos=Vec3(sx,sy,self.terrain.getRealElev(sx,sy))
			else:
				StartPos=Vec3(sx,sy,sz)
			self.setFluidPos(StartPos)
			print 'startpos',StartPos
			self.setLastPos()
			#self.ls()
			
		
	def runAnim(self):
		if self.isMoving is False:
			self.loop("run")
			self.setPlayRate(100,'run')
			self.isMoving = True 
			
	def walkAnim(self):
		if self.isMoving:
			self.stop()
			self.pose("walk",5)
			self.isMoving = False
			
	def setXYZ(self,zadjust):
		(self.rX,self.rY)=self.terrain.getTerrainXY(self.getX(),self.getY())
		if zadjust==0:
			landlev=self.terrain.getRealElev(self.getX(),self.getY())
			if (self.count % 20==0):
				self.terrain.land.setFocalPoint( int(self.rX),int(self.rY) )
				self.count=0
			if ( self.terrain.SEALEVEL > landlev ):
				self.rZ=self.terrain.SEALEVEL
				self.setZ(self.rZ)
			else:			
				self.rZ=landlev
				self.setZ(landlev)
		else:		
			self.rZ=zadjust
			self.setZ(zadjust)
			
		if (int(self.rX)!=self.lastX or int(self.rY)!= self.lastY):
			print int(self.rX),int(self.rY)	
			print "setting focal point"
			self.lastX=int(self.rX)
			self.lastY=int(self.rY)
			
	def setLastPos(self): 	
		self.lastX,self.lastY= self.terrain.getTerrainXY(self.getX(),self.getY());    		
	
		
	def runCommand(self):
		rep=(0,100,0,100)
		com="self.setPos(%s,%s,self.terrain.getRealElev(%s,%s))"
		try:
			eval(com % rep)
		except:	
			print 'could not run function'
			
	def runMagic(self,magic,invars):
		print 'input',magic,invars,self.user.MYDIR
		(magic,status)=self.getMagic(magic,invars)
		if status=='GOOD':
			try:
				exec magic
			except:
				print 'could not run magic'
		else:
			print status
				    
	def getMagic(self,magic,invars):
		status='GOOD'
		try:
			mfile=open(self.user.MYDIR+'/magic/'+magic+'.magic','r')
		except:
			status='Could not open magic file'
		if status=='GOOD':	
			try:	
				magic=mfile.read()
			except:
				status='Could not read magic file'
		if invars!=None and status=='GOOD':				
			try:	
				(magv,maginput)=self.getMagicVars(magic)
				magvars={}
				for i in range(len(magv)):
					magvars[magv[i]]=invars[i]	
				
				for i in range(len(maginput)):
					maginput[i]=magvars[maginput[i]]
				magic= magic % tuple(maginput)
			except:
				status='Could not parse magic variables'
		return magic,status		
			
	def getMagicVars(self,magic):		
		magparts=magic.strip('#').split("\n")
		magparts=magparts[0].split('||')
		magv=magparts[0].split(',')
		maginput=magparts[1].split('/')
		return magv,maginput
			

	def turnLFixed(self,amount):
		self.headchange=True
		self.setH(self.getH() + amount)
		self.isMoving=False
	def turnRFixed(self,amount):
		self.headchange=True
		self.setH(self.getH() - amount)
		self.isMoving=False
	def stopTurn(self):	
		self.headchange=True
		self.keyed=False
		
	def stopMove(self):	
		print "stop"
		self.keyed=False
		self.walkAnim()
		
	def forwFixed(self,amount):
		startpos = self.getPos()
		self.runAnim()
		backward = self.getNetTransform().getMat().getRow3(1)
		backward.setZ(0)
		backward.normalize()
		self.setFluidPos(self.getPos() - backward*(amount*5))
		self.setXYZ()
		
	def backFixed(self,amount):
		self.cammove=True
		self.keyed=True
		startpos = self.getPos()
		self.runAnim()
		self.isMoving=True
		backward = self.getNetTransform().getMat().getRow3(1)
		backward.setZ(0)
		backward.normalize()
		self.setFluidPos(self.getPos() + backward*(amount*5))
		self.setXYZ()
		
	def turnL(self,elapsed):
		self.setH(self.getH() + 50*elapsed)
	def turnR(self,elapsed):
		self.setH(self.getH() - 50*elapsed)
	def forw(self,elapsed):
		backward = self.getNetTransform().getMat().getRow3(1)
		backward.setZ(0)
		backward.normalize()
		self.setFluidPos(self.getPos() - backward*(elapsed*5))
	def back(self,elapsed):
		backward = self.getNetTransform().getMat().getRow3(1)
		backward.setZ(0)
		backward.normalize()
		self.setFluidPos(self.getPos() + backward*(elapsed*5))
		    
	def jump(self,elapsed):
		self.setZ(self.getZ() + 10)    
		backward = self.getNetTransform().getMat().getRow3(1)
		backward.setZ(0)
		backward.normalize()
		self.setFluidPos(self.getPos() - backward*(elapsed*100))

