__author__='sigilis.com'
__version__='0.2'
from pandac.PandaModules import QueuedConnectionManager, QueuedConnectionListener
from pandac.PandaModules import QueuedConnectionReader, ConnectionWriter
from pandac.PandaModules import PointerToConnection, NetAddress, NetDatagram
from direct.task.Task import Task, TaskManager
from direct.showbase.DirectObject import DirectObject
from direct.distributed.PyDatagram import PyDatagram
from direct.distributed.PyDatagramIterator import PyDatagramIterator

from direct.gui.DirectGui import *
import sys

import threading
from datetime import datetime


######################################3
##
## Config
##

PORT = 9100

######################################3
##
## Defines
##

MSG_NONE            	= 0
MSG_AUTH           	= 1
MSG_AUTH_RESP       	= 2
MSG_AUTH_ACK       	= 3
MSG_CHAT           	= 4
MSG_POS           	= 4



class TwoWay(DirectObject):        
	def __init__(self,user):
		self.lastConnection = None
		self.user=user
		self.OTHERS = {}
		self.EstablishedOTHERS = {}
		#install msg handlers
		self.Handlers = {
				    MSG_AUTH           : self.msgAuth,
				    MSG_AUTH_RESP      : self.msgAuthResponse,
				    MSG_AUTH_ACK       : self.msgAuthAcknowledged,
				    MSG_CHAT 	       : self.msgChat,
				    MSG_POS	       : self.msgPos
				    }
		#Thread.__init__(self,name='TwoWay Thread',target=self.StartServer)
		#Thread.start(self)
		self.StartServer()
		
	def StartServer(self):	
	
		# Deals with the basic network stuff
		self.cManager = QueuedConnectionManager()
		
		# Listens for new connections and queue's them
		self.cListener = QueuedConnectionListener(self.cManager, 0)
	
		# Reads data send to the server
		self.cReader = QueuedConnectionReader(self.cManager, 0)
	
		# Writes / sends data to the client
		self.cWriter = ConnectionWriter(self.cManager,0)
	
		# open a server socket on the given port. Args: (port,timeout)
		self.tcpSocket = self.cManager.openTCPServerRendezvous(PORT, 1)
		
		print self.tcpSocket
	
		# Tell the listener to listen for new connections on this socket
		self.cListener.addConnection(self.tcpSocket)
	
	
		# Start Listener task
		#self.taskMgr=TaskManager()
		#self.taskMgr.add(self.listenTask, "serverListenTask",-40)
		taskMgr.add(self.listenTask, "serverListenTask",-40)
		
		# Start Read task
		#self.taskMgr.add(self.readTask, "serverReadTask", -39)
		taskMgr.add(self.readTask, "serverReadTask", -39)
	
		#self.taskMgr.run()
		#taskMgr.run()

	def listenTask(self, task):
		"""
		Accept new incoming connections from the client
		"""
		# Run this task after the dataLoop
		# If there's a new connection Handle it
		#print 'listen task'
		if self.cListener.newConnectionAvailable():
			rendezvous = PointerToConnection()
			netAddress = NetAddress()
			newConnection = PointerToConnection()
			if self.cListener.getNewConnection(rendezvous,netAddress,newConnection):
				newConnection = newConnection.p()
				# tell the Reader that there's a new connection to read from
				self.cReader.addConnection(newConnection)
				self.OTHERS[newConnection] = netAddress.getIpString()
				self.lastConnection = newConnection
				print "Got a connection!"
			else:
				print "getNewConnection returned false"
		return Task.cont


	def readTask(self, task):
		"""
		If there's any data received from the clients,
		get it and send it to the handlers.
		"""
		#print 'read task front'
        	while 1:
			(datagram, data, msgID) = self.nonBlockingRead(self.cReader)
            		if msgID is MSG_NONE:
		    		# got nothing todo
				break
			else:
				# Got a datagram, handle it
				self.handleDatagram(data, msgID,datagram.getConnection())
                
		#if task.frame % 20==0:
		#print 'read task end'
		return Task.cont

	def nonBlockingRead(self,qcr):
		"""
		Return a datagram iterator and type if data is available on the
		queued connection reader
		"""
		if self.cReader.dataAvailable():
			datagram = NetDatagram()
			if self.cReader.getData(datagram):
				data = PyDatagramIterator(datagram)
				msgID = data.getUint16()
			else:
				data = None
				msgID = MSG_NONE
		else:
			datagram = None
			data = None
			msgID = MSG_NONE
		# Note, return datagram to keep a handle on the data
		return (datagram, data, msgID)

	def handleDatagram(self, data, msgID, client):
		"""
		Check if there's a handler assigned for this msgID.
		Since we dont have case statements in python,
		I'm using a dictionary to avoid endless elif statements.
		"""
		if msgID in self.Handlers.keys():
			self.Handlers[msgID](msgID,data,client)
		else:
			print "Unknown msgID: %d" % msgID
			print data
		return
		
	def sendMsgAuth(self,remAddr,remAddrLocal):
		print 'sending MsgAuth'
		if remAddr==self.user.myIP:
			remAddr=remAddrLocal
		self.authThread=threading.Thread(	name='twoWayAuthThread'+str(datetime.now()),
							target=self.threadAuth,kwargs={'remAddr':remAddr,'remAddrLocal':remAddrLocal})
		self.authThread.start()
		print 'thread started'	
		
	def threadAuth(self,remAddr,remAddrLocal):	
		AuthConn = self.cManager.openTCPClientConnection(remAddr, PORT,500)
		
		if AuthConn:
			print 'got Connection'
			self.cReader.addConnection(AuthConn)
			pkg = PyDatagram()
			pkg.addUint16(MSG_AUTH)
			pkg.addString(self.user.username)
			pkg.addString(self.user.modelType)
			pkg.addString(self.user.remoteURL)
			pkg.addUint16(self.user.sector)
			pkg.addFloat32(self.user.avatar.getX())
			pkg.addFloat32(self.user.avatar.getY())
			pkg.addFloat32(self.user.avatar.getZ())
			pkg.addString(self.user.myIP)
			pkg.addString(self.user.localIP)
			
			self.cWriter.send(pkg,AuthConn)
		else:	
			print 'could not connect to %s' % remAddr
		print 'finished msg auth'	
			
	def msgAuth(self, msgID, data, clientConn):
		username = data.getString()
		modelType= data.getString()
		remoteURL= data.getString()
		sector = data.getUint16()
		posX=data.getFloat32()
		posY=data.getFloat32()
		posZ=data.getFloat32()
		client=data.getString()
		localIP=data.getString()
		other={	'username':username,
			'modelType':modelType,
			'remoteURL':remoteURL,
			'sector':sector,
			'posX':posX,
			'posY':posY,
			'posZ':posZ,
			'client':client,
			'localIP':localIP		
			}
		print username,"trying to login"
		print "logging from...%s" % str(clientConn.getAddress())
		if not self.EstablishedOTHERS.has_key(username):
			self.EstablishedOTHERS[username] = {'on':1,'ip':clientConn.getAddress(),'client':clientConn}	
		else:
			if self.EstablishedOTHERS[username]['client'] != clientConn:
				self.EstablishedOTHERS[username]['client'] = clientConn
		if not self.user.others.has_key(username):
			self.user.addOther(other)
		else:
			self.user.others[username].avatar.setFluidPos(posX,posY,posZ)
		print 'OTHERS',self.EstablishedOTHERS	
		pkg = PyDatagram()
		pkg.addUint16(MSG_AUTH_RESP)
		pkg.addString(self.user.username)
		pkg.addUint16(self.user.sector)
		pkg.addFloat32(self.user.avatar.getX())
		pkg.addFloat32(self.user.avatar.getY())
		pkg.addFloat32(self.user.avatar.getZ())
	
		self.cWriter.send(pkg,clientConn)
		
	def msgAuthResponse(self, msgID, data, client):
		username = data.getString()
		sector = data.getUint16()
		posX=data.getFloat32()
		posY=data.getFloat32()
		posZ=data.getFloat32()
		print username,"responding"
		print "responding from...%s" % client.getAddress()
		if not self.EstablishedOTHERS.has_key(username):
			self.EstablishedOTHERS[username] = {'on':1,'ip':client.getAddress(),'client':client}		
		if self.user.others.has_key(username):
			self.user.others[username].avatar.setFluidPos(posX,posY,posZ)
		pkg = PyDatagram()
		pkg.addUint16(MSG_AUTH_ACK)
		pkg.addUint32(1)
	
		self.cWriter.send(pkg,client)		
		
	def msgAuthAcknowledged(self, msgID, data, client):
		flag=data.getUint32()
		print 'acknowledged'
		
	def msgChat(self, msgID, data, client):
	    	print "ChatMsg: %s" % data.getString()	
		
	def sendMsgPos(self):
		#print 'others',self.user.others
		for other in self.EstablishedOTHERS.iteritems():	
			username,data=other
			#print 'username,data',username,data
			if self.user.sector==self.user.others[username].sector:
				pkg = PyDatagram()
				pkg.addUint16(MSG_POS)
				pkg.addString(self.user.username)
				pkg.addUint16(self.user.sector)
				pkg.addFloat32(self.user.avatar.getX())
				pkg.addFloat32(self.user.avatar.getY())
				pkg.addFloat32(self.user.avatar.getZ())
				pkg.addFloat32(self.user.avatar.getH())
				self.cWriter.send(pkg,data['client'])
				#print 'sending position to', username
			
	def msgPos(self, msgID, data, client):
		username = data.getString()
		sector = data.getUint16()
		posX=data.getFloat32()
		posY=data.getFloat32()
		posZ=data.getFloat32()
		heading=data.getFloat32()			
		if self.user.sector==sector:
			self.user.others[username].avatar.setFluidPos(posX,posY,posZ)
			self.user.others[username].avatar.setH(heading)
