__author__='sigilis.com'
__version__='0.2'
from direct.showbase.DirectObject import DirectObject
from pandac.PandaModules import Vec3,Vec4,BitMask32
from direct.gui.OnscreenText import OnscreenText
from pandac.PandaModules import PandaNode,NodePath,Camera,TextNode               
from pandac.PandaModules import CollisionTraverser,CollisionNode
from pandac.PandaModules import CollisionHandlerQueue,CollisionRay
from pandac.PandaModules import DirectionalLight,AmbientLight,PointLight
from direct.task.Task import Task

import random, sys, os, math

from BasicFunctions import *
from User import User
from Terrain import Terrain

VSCALE=50000.0
HSCALE=1000.0                                                                                               
VIZRAD=100000


SIGILISRADIUS=1236077.34

class World(DirectObject):
	def __init__(self,me,MYDIR):
		#global STARTX,STARTY,VSCALE,HSCALE,VIZRAD,MYDIR
		self.me=me
		#self.keyMap = {"help":0,"edit":0}
		base.win.setClearColor(Vec4(0,0,0,1))
		# Post the instructions
		self.radius=SIGILISRADIUS
		self.title = addTitle("Sigilis")
		self.instructions='off'		      
		self.InstuctionsHUD('on')

		self.redSunLight = PointLight('RedSun')
		self.redSunLight.setColor(Vec4(1,0.7,0.7,1))
		self.grnSunLight = PointLight('GreenSun')
		self.grnSunLight.setColor(Vec4(0.7,1,0.7,1))
		self.bluSunLight = PointLight('BlueSun')
		self.bluSunLight.setColor(Vec4(0.7,0.7,1,1))
		
		
		
		rsunLightNP = render.attachNewNode( self.redSunLight.upcastToPandaNode() )
		gsunLightNP = render.attachNewNode( self.grnSunLight.upcastToPandaNode() )
		bsunLightNP = render.attachNewNode( self.bluSunLight.upcastToPandaNode() )
		
		render.setLight(rsunLightNP)
		render.setLight(gsunLightNP)
		render.setLight(bsunLightNP)
		
		# Create Ambient Light
		ambientLight = AmbientLight( 'ambientLight' )
		#ambientLight.setColor( Vec4( 1, 1, 1, .5 ) )
		ambientLight.setColor( Vec4( 1, 1, 1, .1 ) )
		ambientLightNP = render.attachNewNode( ambientLight.upcastToPandaNode() )
		#render.setLight(ambientLightNP)
		
		#create terrain
		self.Terra=Terrain(	self,
					sector = self.me.sector,
					hscale = HSCALE,
					vscale = VSCALE,
					vizrad = VIZRAD,
					MYDIR = MYDIR)
	
		self.Terra.setSuns()
		
		#create Avatar
		self.me.setAvatar(sx=self.me.startX,sy=self.me.startY,
				sz=self.me.startZ,terrain=self.Terra,eyeheight=4.2)
				
		#login
		self.me.setClient()   
		if self.me.twoway==False:
			self.me.startTwoWay()
		#if self.me.client.connected==True:
		#	print 'connected to server, starting 2way'
			
		
		# Accept the control keys for movement and rotation
		self.accept("control-q", self.signOff)
		self.accept("h", self.showHideHelp)
		
		# Game state variables
		self.prevtime = 0
		
		base.disableMouse()

	def signOff(self):
		if self.me.client.connected==True:
			print 'signing off...'
			self.me.client.sendMsgDisconnectReq()			
		self.me.killAvatar()
		self.me.killClient()
		self.me.killTwoway()
		from libs.StartSplash import StartSplash
		ss=StartSplash(self.me,self.me.MYDIR)
	
	def showHideHelp(self):
		if (self.instructions=='on'):
			self.InstuctionsHUD('off')
			self.instructions='off'
		else:	
			self.InstuctionsHUD('on')
			self.instructions='on'
		
	def reviveHelp(self):
		self.accept("h", self.showHideHelp)
	
	def InstuctionsHUD(self,onOff):
		if (onOff=='on'):
			self.inst= [addInstructions(0.95, "[Control-Q]: Quit",self.me.widescreen),
				addInstructions(0.90, "[Left Arrow]: Rotate Right",self.me.widescreen),
				addInstructions(0.85, "[Right Arrow]: Rotate Left",self.me.widescreen),
				addInstructions(0.80, "[Up Arrow]: Forward",self.me.widescreen),
				addInstructions(0.75, "[Down Arrow]: Back",self.me.widescreen),
				addInstructions(0.70, "[A]: Rotate Camera Right",self.me.widescreen),
				addInstructions(0.65, "[S]: Rotate Camera Left",self.me.widescreen),
				addInstructions(0.60, "[Control-F]: Toggle Flying Mode",self.me.widescreen),
				addInstructions(0.55, "[1]: Toggle First Person",self.me.widescreen),
				addInstructions(0.50, "[2]: Cycle Through Overhead Views",self.me.widescreen),
				addInstructions(0.45, "[H]: Toggle Help",self.me.widescreen) ]
			self.instructions='on'
		else:
			for i in range(0,11):
				self.inst[i].destroy()
			self.instructions='off'

		
