__author__='sigilis.com'
__version__='0.2.6'
from pandac.PandaModules import Filename,InternalName
from pandac.PandaModules import Texture,TextureStage
from pandac.PandaModules import TexGenAttrib
from pandac.PandaModules import Vec3
from pandac.PandaModules import Geom, GeomNode, GeomTrifans, GeomTristrips
from pandac.PandaModules import GeomVertexReader, GeomVertexWriter
from pandac.PandaModules import GeomVertexRewriter, GeomVertexData
from pandac.PandaModules import GeomVertexArrayFormat, GeomVertexFormat
from pandac.PandaModules import NodePath,BitMask32

from SigilisModel import SigilisModel
import math

DEFAULTCOLUMN='textures/columns/column1.png'

class Column(SigilisModel): 
	def __init__(self,data=None,user=None):
		if data !=None:
			SigilisModel.__init__(self,user=user,data=data,modeltype='Column',DEFAULTU=Texture.WMClamp,DEFAULTV=Texture.WMClamp)
			if self.good==True:
				if data.has_key('heading'):
					self.heading=(float(data['heading'])/360)*(2*math.pi)
				else:
					self.heading=False
				self.vertices=int(data['vertices'])
				
				formatArray=GeomVertexArrayFormat()
				formatArray.addColumn(InternalName.make("drawFlag"), 1, Geom.NTUint8, Geom.COther)
		
				format=GeomVertexFormat(GeomVertexFormat.getV3n3cpt2())
				format.addArray(formatArray)
				self.format=GeomVertexFormat.registerFormat(format)
		
				self.vdata=GeomVertexData("body vertices", self.format, Geom.UHStatic)
	
				self.makeCylinder()
				if self.tile=='SHINY':
					shinyfile='textures/columns/'+self.nodeName.replace(':','.')+'shiny.jpg'
					camnow=base.camera.getPos()
					base.camera.setPos(self.x,self.y,self.z+self.height/2)
					base.saveSphereMap(shinyfile, size = 128)
					base.camera.setPos(camnow)
					self.tile=shinyfile
					self.nPath.setTexGen(TextureStage.getDefault(), TexGenAttrib.MEyeSphereMap)
	
				if self.tile_remote==1:
					self.checkRemoteTexture(self.nPath,'textures/columns/',self.tile,
								DEFAULTCOLUMN,self.wrapU,self.wrapV)
				else:
		
					colTexture=loader.loadTexture(self.tile)
					colTexture.setWrapU(self.wrapU)
					colTexture.setWrapV(self.wrapV)
					#self.nPath.setTexGen( TextureStage.getDefault( ), TexGenAttrib.MWorldPosition )
					#self.nPath.setTexScale( TextureStage.getDefault( ), .25, 1 )
					self.nPath.setTexture(colTexture)
				self.nPath.reparentTo(render.find('Objects/Columns'))
				self.nPath.setPos(self.x,self.y,self.z)
			else:
				del(self)
				
	def editScreen(self):
		self.user.avatar.accepting=False
		if self.user.widescreen=='yes':
			xpos=-1.2
		else:
			xpos=-1.0
		self.editFrame = DirectFrame(frameSize = (-.3,.3,-.3,.3), frameColor=(.3,.2,.1,1),
						pos = (xpos,0, .5)) 
		self.editFrame.reparentTo(aspect2d)
		info = DirectLabel(parent=self.editFrame,text=self.nodeName,
				      pos=(0,0,.25), scale = .05)
		x = DirectLabel(parent=self.editFrame,text='X Pos:',
				      pos=(-.2,0,.15), scale = .04)
		self.xEntry = DirectEntry(parent=self.editFrame,initialText=str(self.nPath.getX()),
				      pos=(-.11,0,.15), scale = .04)
		y = DirectLabel(parent=self.editFrame,text='Y Pos:',
				      pos=(-.2,0,.05), scale = .04)
		self.yEntry = DirectEntry(parent=self.editFrame,initialText=str(self.nPath.getY()),
				      pos=(-.11,0,.05), scale = .04)
		z = DirectLabel(parent=self.editFrame,text='Z Pos:',
				      pos=(-.2,0,-.05), scale = .04)
		self.zEntry = DirectEntry(parent=self.editFrame,initialText=str(self.nPath.getZ()),
				      pos=(-.11,0,-.05), scale = .04)
		if (self.owner==self.user.username):	      
			setButton = DirectButton(parent=self.editFrame,text='Set Pos',
						  pos=(-.15,0,-.20), scale = .04)
			updateButton = DirectButton(parent=self.editFrame,text='Update Pos',
						  pos=(.15,0,-.20), scale = .04,command=self.updatePos)
		else:
			owner = DirectLabel(parent=self.editFrame,text='Owner: '+self.owner,
				      pos=(-.2,0,-.20), scale = .04)	
		exitButton = DirectButton(parent=self.editFrame,text='EXIT',
					  pos=(0,0,-.26), scale = .04,command=self.exitEdit)			 
					  	
	def getVarSpec(self):
		return {'x':['float',0,'text','X position'],'y':['float',0,'text','Y position'],'z':['float',0,'text','Z position'],'width':['float',1,'text','Column width'],'height':['float',1,'text','Column height'],
			'vertices':['int',1,'text','Vertices'],'heading':['float',1,'text','Heading (0-360)'],
			'tile':['str',2,'text/file','Column texture local relative path or URL|or, Upload File'],'tile_remote':['bool',2,'radio','Is texture remote?'],
			'tile_scale':['float',1,'text','Texture Scale'],'reqtosubmit':['str',0,'hidden','x,y,z,width,height,tile'],
			'defaults':['str',0,'hidden','heading=0,tile_scale=1']}		
		
	def makeCircle(self, offset=Vec3(0,0,0), top=True,direction=1):
		circleGeom=Geom(self.vdata)
	
		vertWriter=GeomVertexWriter(self.vdata, "vertex")
		normalWriter=GeomVertexWriter(self.vdata, "normal")
		colorWriter=GeomVertexWriter(self.vdata, "color")
		uvWriter=GeomVertexWriter(self.vdata, "texcoord")
		drawWriter=GeomVertexWriter(self.vdata, "drawFlag")
	
		#make sure we start at the end of the GeomVertexData so we dont overwrite anything
		#that might be there already
		startRow=self.vdata.getNumRows()
		#print 'startRow',startRow
		vertWriter.setRow(startRow)
		colorWriter.setRow(startRow)
		uvWriter.setRow(startRow)
		normalWriter.setRow(startRow)
		drawWriter.setRow(startRow)
	
		angle=2*math.pi/self.vertices
		if self.heading==False:
			currAngle=angle
		else:
			currAngle=angle+self.heading
	
		for i in range(self.vertices+1):
			position=Vec3(self.width*math.cos(currAngle)+offset.getX(), self.width*math.sin(currAngle)+offset.getY(),offset.getZ())
			vertWriter.addData3f(position)
			#uvWriter.addData2f(position.getX()/2.0+0.5,position.getY()/2.0+0.5)
			
			if startRow==0:
				uvWriter.addData2f(1-float(i)/(float(self.vertices)),startRow/self.vertices)
				#print 'rowinfo',i,position,1-float(i)/(float(self.vertices)),startRow/self.vertices,i+startRow
			else:	
				uvWriter.addData2f(float(i)/(float(self.vertices)),startRow/self.vertices)
				#print 'rowinfo',i,position,float(i)/(float(self.vertices)),startRow/self.vertices,i+startRow
			colorWriter.addData4f(1.0, 1.0, 1.0, 1.0)
			position.setZ(position.getZ()*direction)
			position.normalize()
			normalWriter.addData3f(position)
			
			#at default Opengl only draws "front faces" (all shapes whose vertices are arranged CCW). We
			#need direction so we can specify which side we want to be the front face
			currAngle+=angle*direction
	
		circle=GeomTrifans(Geom.UHStatic)
		circle.addConsecutiveVertices(startRow, self.vertices)
		circle.closePrimitive()
	
		circleGeom.addPrimitive(circle)
		
		return circleGeom
		
	def makeCylinder(self):
		bottomCircleGeom=self.makeCircle(Vec3(0,0,0),False,-1)
		topCircleGeom=self.makeCircle(Vec3(0,0,self.height))
		
		cylinderGeom=Geom(self.vdata)	
		
		for i in range(self.vertices):
			body=GeomTristrips(Geom.UHStatic)
			body.addVertex(i)		
			body.addVertex(self.vertices*2+1-i)
			body.addVertex(i+1)
			
			body.closePrimitive()
			cylinderGeom.addPrimitive(body)
			
		for i in range(1,self.vertices+1):
			body=GeomTristrips(Geom.UHStatic)
			body.addVertex(i)		
			body.addVertex(self.vertices*2+2-i)
			body.addVertex(self.vertices*2+1-i)
			
			body.closePrimitive()
			cylinderGeom.addPrimitive(body)
		
		cylinderGeom.copyPrimitivesFrom(bottomCircleGeom)
		cylinderGeom.copyPrimitivesFrom(topCircleGeom)
		
		cylinderGeom.decomposeInPlace()
		## maybe later???
		cylinderGeom.unifyInPlace(16,False)
		
		cylGeomNode=GeomNode("ColGeom")
		cylGeomNode.addGeom(cylinderGeom)
		cylGeomNode.setIntoCollideMask(BitMask32.bit(19))
			
				
		self.nPath.attachNewNode(cylGeomNode)
		
