__author__='sigilis.com'
__version__='0.2.6'
from pandac.PandaModules import Filename,InternalName
from pandac.PandaModules import Texture,TextureStage
from pandac.PandaModules import TexGenAttrib
from pandac.PandaModules import Vec3
from pandac.PandaModules import Geom, GeomNode, GeomTrifans, GeomTristrips
from pandac.PandaModules import GeomVertexReader, GeomVertexWriter
from pandac.PandaModules import GeomVertexRewriter, GeomVertexData
from pandac.PandaModules import GeomVertexArrayFormat, GeomVertexFormat
from pandac.PandaModules import NodePath

from SigilisModel import SigilisModel

DEFAULTFLOOR='textures/floors/checkerboard.png'

class Floor(SigilisModel): 
	def __init__(self,data=None,user=None):
		if data !=None:
			SigilisModel.__init__(self,user=user,data=data,modeltype='Floor',DEFAULTU=Texture.WMRepeat,DEFAULTV=Texture.WMRepeat)
			if self.good==True:
				self.style=data['style']
				self.heading=float(data['heading'])
				formatArray=GeomVertexArrayFormat()
				formatArray.addColumn(InternalName.make("drawFlag"), 1, Geom.NTUint8, Geom.COther)
		
				format=GeomVertexFormat(GeomVertexFormat.getV3n3cpt2())
				format.addArray(formatArray)
				self.format=GeomVertexFormat.registerFormat(format)
	
				self.floordata=GeomVertexData("inner vertices", self.format, Geom.UHStatic)
	
				self.floor=self.nPath			
				self.floor.node().setIntoCollideMask(GeomNode.getDefaultCollideMask())
				
				floorGeom=self.makeFloor()
				
				if self.tile_remote==1:
					self.checkRemoteTexture(self.floor,'textures/floors/',self.tile,
								DEFAULTFLOOR,self.wrapU,self.wrapV)
				else:	
					floorTexture=loader.loadTexture(self.tile)
				
					floorTexture.setWrapU(self.wrapU)
					floorTexture.setWrapV(self.wrapV)
					self.floor.setTexRotate(TextureStage.getDefault(),self.tile_rot)
					self.floor.setTexture(floorTexture)
					
				self.floor.setTwoSided(True)
				self.floor.attachNewNode(floorGeom)
				
				self.floor.reparentTo( render.find('Objects/Floors') )
				self.floor.setH(self.heading)
				self.floor.setPos( self.x,self.y,self.z)
				self.floor.ls()
			else:
				del(self)

	def getVarSpec(self):
		return {'x':['float',0,'text','X position'],'y':['float',0,'text','Y position'],'z':['float',0,'text','Z position'],
			'heading':['float',1,'text','Heading'],'width':['float',2,'text','Floor width'],'length':['float',2,'text','Floor length'],
			'tile':['str',3,'text/file','Floor tile texture local relative path or URL|or, Upload File'],
			'tile_remote':['bool',4,'radio','Is texture remote?'],'tile_scale':['float',5,'text','Tile Scale Factor'],
			'tile_rot':['float',6,'text','Rotate Floor (degrees)'],
			'style':['str',7,'select|square,rectangle','Floor Style'],
			'reqtosubmit':['str',0,'hidden','x,y,z,width,length,tile'],
			'defaults':['str',0,'hidden','heading=0,tile_scale=1,tile_rot=0']}
			
			#not ready:
			#'wrapU':['str',8,'text','Wrap U (optional)'],'wrapV':['str',9,'text','Wrap V (optional)'], 
			
	def makeFloor(self):
		self.floorGeom=Geom(self.floordata)
		self.vertWriter=GeomVertexWriter(self.floordata, "vertex")
		self.normalWriter=GeomVertexWriter(self.floordata, "normal")
		self.colorWriter=GeomVertexWriter(self.floordata, "color")
		self.uvWriter=GeomVertexWriter(self.floordata, "texcoord")
		self.drawWriter=GeomVertexWriter(self.floordata, "drawFlag")
		
		self.vertWriter.setRow(0)
		self.colorWriter.setRow(0)
		self.uvWriter.setRow(0)
		self.normalWriter.setRow(0)
		self.drawWriter.setRow(0)
		
		if self.style=='square' or self.style=='rectangle':
			floor=self.Square()
			
		print floor	
		self.floorGeom.addPrimitive(floor)
		self.floorGeom.decomposeInPlace()
		#wallGeom.unifyInPlace(4,False)
		floorGeomNode=GeomNode("FloorGeom:"+str(self.x)+'-'+str(self.y)+'-'+str(self.z))
		floorGeomNode.addGeom(self.floorGeom)
			
		return floorGeomNode	
			
	def Square(self):
		for i in range(2):
			for j in range(2):
				position=Vec3(j*self.width, i*self.length,0)
				self.vertWriter.addData3f(position)
				self.uvWriter.addData2f(j*self.width/self.tile_scale, i*self.length/self.tile_scale)	
				self.colorWriter.addData4f(1.0, 1.0, 1.0, 0.0)
				position.normalize()
				self.normalWriter.addData3f(position)	
				
		floor=GeomTrifans(Geom.UHStatic)
		floor.addVertex(0)		
		floor.addVertex(1)
		floor.addVertex(3)
		floor.addVertex(2)	
		floor.closePrimitive()
		return floor
				
