__author__='sigilis.com'
__version__='0.2'
from pandac.PandaModules import Filename,InternalName
from pandac.PandaModules import Texture,TextureStage,MovieTexture
from pandac.PandaModules import Vec3
from pandac.PandaModules import Geom, GeomNode, GeomTrifans, GeomTristrips
from pandac.PandaModules import GeomVertexReader, GeomVertexWriter
from pandac.PandaModules import GeomVertexRewriter, GeomVertexData
from pandac.PandaModules import GeomVertexArrayFormat, GeomVertexFormat
from pandac.PandaModules import NodePath,BitMask32
from pandac.PandaModules import MovieAudio,MovieAudioCursor,AudioSound
from pandac.PandaModules import HTTPClient,URLSpec,MovieVideo,Ramfile
from direct.showbase import Audio3DManager
from direct.task.Task import Task

import urllib
import threading
from SigilisModel import SigilisModel

DEFAULTMOVIE='movies/default.mpg'

class MovieWall(SigilisModel): 
	def __init__(self,data=None,user=None):
		if data !=None:
			SigilisModel.__init__(self,user=user,data=data,modeltype='MovieWall')
			if self.good==True:
				self.movie=data['movie']                       
				self.movie_remote=int(data['movie_remote'])
				self.movie_scale=float(data['movie_scale'])
				self.movieheight=float(data['movieheight'])
				self.moviewidth=float(data['moviewidth'])
				self.heading=float(data['heading'])
				
				formatArray=GeomVertexArrayFormat()
				formatArray.addColumn(InternalName.make("drawFlag"), 1, Geom.NTUint8, Geom.COther)
		
				format=GeomVertexFormat(GeomVertexFormat.getV3n3cpt2())
				format.addArray(formatArray)
				self.format=GeomVertexFormat.registerFormat(format)
	
				self.screendata=GeomVertexData("inner vertices", self.format, Geom.UHStatic)
	
				self.screen=self.nPath			
				self.screen.node().setIntoCollideMask(BitMask32.bit(19))
				
				screenGeom=self.makeScreen()
				
				if self.movie_remote==1:
					self.localmovie='movies/'+self.movie.replace('://','.').replace('/','.')
					lfile=Filename.binaryFilename(self.localmovie)
					if lfile.exists():
						screenTexture=loader.loadTexture(self.localmovie)
					else:
						screenTexture=loader.loadTexture(DEFAULTMOVIE)
						getmovie=threading.Thread(name='GetMovie'+str(self.__x)+'-'+str(self.__y)+'-'+str(self.__z),target=self.getMovie)
						getmovie.start()
						#screenTexture=self.getRemoteMovie()
				else:	
					screenTexture=loader.loadTexture(self.movie)
				screenAudio=MovieAudio()
				screenAudio.get(Filename(self.movie))
				screenAudioC=MovieAudioCursor(screenAudio)
				self.screenAudioSound=loader.loadSfx(self.movie)
				#soundEffectsMgr = base.sfxManagerList[0]
				#audio3d = Audio3DManager.Audio3DManager(base.sfxManagerList[0], self.screen)
				#screenAudioSound = audio3d.loadSfx(self.movie)
				
				screenTexture.synchronizeTo(self.screenAudioSound)
				self.screenAudioSound.setLoop(True)
				self.screenAudioSound.setVolume(1)
				self.screenAudioSound.play()
					#screenAudioC=screenAudio.open()
				
				screenTexture.setWrapU(Texture.WMClamp)
				screenTexture.setWrapV(Texture.WMClamp)
				self.screen.setTexture(screenTexture)
				#self.screen.setTwoSided(False)
				self.screen.setTwoSided(False)
				self.screen.attachNewNode(screenGeom)
				
				self.screen.reparentTo( render.find('Objects/MovieWalls')  )
				self.screen.setH(self.heading)
				self.screen.setPos( self.x,self.y,self.z)
				
				self.screen.ls()
				taskMgr.add(self.play,"playTask")
			else:
				del(self)
	
	def getVarSpec(self):
		return {'x':['float',0,'text','X position'],'y':['float',0,'text','Y position'],'z':['float',0,'text','Z position'],
			'heading':['float',1,'text','Heading'],'width':['float',2,'text','Screen width'],'height':['float',2,'text','Screen height'],
			'movie':['str',3,'text/file','Movie local relative path or URL|or, Upload File'],
			'movie_remote':['bool',4,'radio','Is movie remote?'],'movie_scale':['float',2,'text','Movie scale'],
			'moviewidth':['int',2,'text','Movie width'],'movieheight':['int',2,'text','Movie height'],
			'reqtosubmit':['str',0,'hidden','x,y,z,movie,width,moviewidth,movieheight'],
			'defaults':['str',0,'hidden','heading=0,movie_scale=1']}
			
	def makeScreen(self):
		pow2=[2,4,8,16,32,64,128,256,512,1024,2048,4096]
		self.screenGeom=Geom(self.screendata)
		self.vertWriter=GeomVertexWriter(self.screendata, "vertex")
		self.normalWriter=GeomVertexWriter(self.screendata, "normal")
		self.colorWriter=GeomVertexWriter(self.screendata, "color")
		self.uvWriter=GeomVertexWriter(self.screendata, "texcoord")
		self.drawWriter=GeomVertexWriter(self.screendata, "drawFlag")
		
		self.vertWriter.setRow(0)
		self.colorWriter.setRow(0)
		self.uvWriter.setRow(0)
		self.normalWriter.setRow(0)
		self.drawWriter.setRow(0)
		
		self.width=(self.moviewidth/self.movieheight)*self.height;
		hmatch=wmatch=False
		for p in pow2:
			if p>self.movieheight and hmatch==False:
				self.hpow2=p
				hmatch=True
			if p>self.moviewidth and wmatch==False:
				self.wpow2=p
				wmatch=True
		for i in range(2):
			for j in range(2):
				position=Vec3(j*self.width, 0,i*self.height)
				self.vertWriter.addData3f(position)
				self.uvWriter.addData2f(j*self.moviewidth/self.wpow2, i*self.movieheight/self.hpow2)	
				self.colorWriter.addData4f(1.0, 1.0, 1.0, 0.0)
				position.normalize()
				self.normalWriter.addData3f(position)	
				
		screen=GeomTristrips(Geom.UHStatic)
		screen.addVertex(0)		
		screen.addVertex(1)
		screen.addVertex(3)
		screen.addVertex(2)	
		screen.addVertex(0)		
		screen.closePrimitive()
			
		print screen	
		self.screenGeom.addPrimitive(screen)
		self.screenGeom.decomposeInPlace()
		#wallGeom.unifyInPlace(4,False)
		screenGeomNode=GeomNode("Screen:"+str(self.x)+'-'+str(self.y)+'-'+str(self.z))
		screenGeomNode.addGeom(self.screenGeom)
			
		return screenGeomNode	
		
	def getMovie(self):
		self.movieLoaded=False
		#(loadfilename,loadheaders)=urllib.urlretrieve(self.movie,self.localmovie,self.movieWatch)
		(loadfilename,loadheaders)=urllib.urlretrieve(self.movie,self.localmovie)
		self.screen.setTexture(loader.loadTexture(loadfilename))
		
	def movieWatch(self,count=0,size=0,total=0):
		if self.movieLoaded==False:
			print 'moviecount',count*size,total
			if total>1000000:
				if count*size>999999:
					self.screen.setTexture(loader.loadTexture(self.localmovie))
					self.movieLoaded=True
			else:
				if count*size>total/4:
					self.screen.setTexture(loader.loadTexture(self.localmovie))
					self.movieLoaded=True
					
	def getRemoteMovie(self):
		movClient=HTTPClient()
		movURL=URLSpec(self.movie)
		print 'client,url',movClient,movURL
		movDoc=movClient.getDocument(movURL)
		#movRam=Ramfile()
		#movDoc.downloadToRam(movRam)
		print 'movdoc',movDoc,movDoc.isValid(),movDoc.getStatusCode()
		#movVid=MovieVideo.get(movDoc.getFilename())
		#print 'movvid',movVid
		#movTex=loader.loadTexture(movVid.getFilename())
		#movTex=Texture.read(movVid.getFilename())
		content=movDoc.readFile(False)
		print 'content',content
		movTex=MovieTexture()
		#movTex. Texture.TT2dTexture,
		#return Texture.read(movDoc.getFilename())
		return movTex
		
	def play(self,task):
		distance=(self.screen.getPos()-self.user.avatar.getPos()).length()
		if distance>200.0:
			self.screenAudioSound.setVolume(0)
		elif distance==0:
			self.screenAudioSound.setVolume(1.0)
		else:
			self.screenAudioSound.setVolume((200-distance)/200)	
		return Task.cont
		 
					  	
			
