__author__='sigilis.com'
__version__='0.2.1'
from pandac.PandaModules import Filename,InternalName
from pandac.PandaModules import Texture,TextureStage
from pandac.PandaModules import TexGenAttrib
from pandac.PandaModules import Vec3
from pandac.PandaModules import Geom, GeomNode, GeomTrifans, GeomTristrips
from pandac.PandaModules import GeomVertexReader, GeomVertexWriter
from pandac.PandaModules import GeomVertexRewriter, GeomVertexData
from pandac.PandaModules import GeomVertexArrayFormat, GeomVertexFormat
from pandac.PandaModules import NodePath, BitMask32

from SigilisModel import SigilisModel

class Road(SigilisModel): 
	def __init__(self,data=None,user=None):
		if data !=None:
			SigilisModel.__init__(self,user=user,data=data,modeltype='Road')
			if self.good==True:
				self.z2=float(data['z2'])
				self.style=data['style']
				self.heading=float(data['heading'])
				
				formatArray=GeomVertexArrayFormat()
				formatArray.addColumn(InternalName.make("drawFlag"), 1, Geom.NTUint8, Geom.COther)
		
				format=GeomVertexFormat(GeomVertexFormat.getV3n3cpt2())
				format.addArray(formatArray)
				self.format=GeomVertexFormat.registerFormat(format)
	
				self.roaddata=GeomVertexData("inner vertices", self.format, Geom.UHStatic)
	
				self.road=self.nPath		
				self.road.node().setIntoCollideMask(BitMask32.bit(20))
				
				roadGeom=self.makeRoad()
				
				roadTexture=loader.loadTexture(self.tile)
				
				roadTexture.setWrapU(Texture.WMRepeat)
				roadTexture.setWrapV(Texture.WMClamp)
				self.road.setTexture(roadTexture)
				self.road.setTwoSided(True)
				self.road.attachNewNode(roadGeom)
				
				self.road.reparentTo( render.find('Objects/Roads') )
				self.road.setH(self.heading-90)
				self.road.setPos( self.x,self.y,self.z)
				self.road.ls()
			else:
				del(self)

	def makeRoad(self):
		self.roadGeom=Geom(self.roaddata)
		self.vertWriter=GeomVertexWriter(self.roaddata, "vertex")
		self.normalWriter=GeomVertexWriter(self.roaddata, "normal")
		self.colorWriter=GeomVertexWriter(self.roaddata, "color")
		self.uvWriter=GeomVertexWriter(self.roaddata, "texcoord")
		self.drawWriter=GeomVertexWriter(self.roaddata, "drawFlag")
		
		self.vertWriter.setRow(0)
		self.colorWriter.setRow(0)
		self.uvWriter.setRow(0)
		self.normalWriter.setRow(0)
		self.drawWriter.setRow(0)
		
		road=self.Vertexes()
			
		print road	
		self.roadGeom.addPrimitive(road)
		
		self.roadGeom.decomposeInPlace()
		#wallGeom.unifyInPlace(4,False)
		roadGeomNode=GeomNode("RoadGeom:"+str(self.x)+'-'+str(self.y)+'-'+str(self.z))
		roadGeomNode.addGeom(self.roadGeom)
			
		return roadGeomNode	
			
	def Vertexes(self):
		for i in range(2):
			for j in range(2):
				position=Vec3(j*self.length, 0,j*(self.z2-self.z)-i)
				self.vertWriter.addData3f(position)
				self.uvWriter.addData2f(j*self.length, 0)	
				self.colorWriter.addData4f(1.0, 1.0, 1.0, 0.0)
				position.normalize()
				self.normalWriter.addData3f(position)	
			
			for j in range(2):
				position=Vec3(j*self.length, self.width,j*(self.z2-self.z)-i)
				self.vertWriter.addData3f(position)
				self.uvWriter.addData2f(j*self.length, 1)	
				self.colorWriter.addData4f(1.0, 1.0, 1.0, 0.0)
				position.normalize()
				self.normalWriter.addData3f(position)		
			
				
		road=GeomTrifans(Geom.UHStatic)
		#road=GeomTriangles()
		road.addVertex(0)		
		road.addVertex(1)
		road.addVertex(3)
		road.addVertex(2)
		
		self.vertWriter.setRow(1)
		self.colorWriter.setRow(1)
		self.uvWriter.setRow(1)
		self.normalWriter.setRow(1)
		self.drawWriter.setRow(1)
		road.addVertex(4)		
		road.addVertex(5)
		road.addVertex(1)
		road.addVertex(0)
		
		self.vertWriter.setRow(2)
		self.colorWriter.setRow(2)
		self.uvWriter.setRow(2)
		self.normalWriter.setRow(2)
		self.drawWriter.setRow(2)
		road.addVertex(2)		
		road.addVertex(3)
		road.addVertex(7)
		road.addVertex(6)
		
		road.closePrimitive()
		return road
				
	def getVarSpec(self):
		return {'x':['float',0,'text','X position'],'y':['float',0,'text','Y position'],'z':['float',0,'text','Z position'],
			'z2':['float',0,'text','second Z position'],
			'heading':['float',1,'text','Heading (0-360)'],
			'width':['float',2,'text','Road width'],'length':['float',2,'text','Road length'],
			'tile':['str',2,'text/file','Road  texture local relative path or URL|or, Upload File'],
			'tile_remote':['bool',4,'radio','Is texture remote?'],
			'reqtosubmit':['str',0,'hidden','x,y,z,z2,tile,width,length'],
			'defaults':['str',0,'hidden','heading=0']}		
			
