__author__='sigilis.com'
__version__='0.2.1'
from direct.directbase import DirectStart
from pandac.PandaModules import Filename,InternalName
from pandac.PandaModules import GeomVertexArrayFormat, GeomVertexFormat
from pandac.PandaModules import Geom, GeomNode, GeomTrifans, GeomTristrips
from pandac.PandaModules import GeomVertexReader, GeomVertexWriter
from pandac.PandaModules import GeomVertexRewriter, GeomVertexData
from pandac.PandaModules import PerspectiveLens, TextNode
from pandac.PandaModules import TransformState,CullFaceAttrib
from pandac.PandaModules import Light,AmbientLight,Spotlight
from pandac.PandaModules import NodePath
from pandac.PandaModules import Vec3,Vec4,Mat4
from direct.task.Task import Task
from direct.gui.OnscreenText import OnscreenText
from direct.showbase.DirectObject import DirectObject
import math, random, time, sys, os

import urllib
import threading

random.seed()
numPrimitives=0

DEFAULTBARK='textures/trees/barkTexture.jpg'
DEFAULTLEAF='textures/trees/leaf1.png'

class Tree(DirectObject): 
	def __init__(self,data=None):
		if (data !=None):
			self.nodeName="Tree:"+str(data['x'])+'-'+str(data['y'])+'-'+str(data['z'])
			if render.find('Objects/Trees').isEmpty():
				Group=NodePath('Trees')	
				Group.reparentTo(render.find('Objects'))
				Group.setPos(0,0,0)
				
			if render.find('Objects/Trees/'+self.nodeName).isEmpty():   
			
				print 'running tree'
				self.ttype=data['type']
				self.bark=data['bark']
				self.bark_remote=data['bark_rem']
				self.leaf=data['leaf']
				self.leaf_remote=data['leaf_rem']
				self.leafModel=data['leafmodel']
				self.heading=data['heading']
				self.scale=data['scale']
				print 'leafmod',self.leafModel		
				self.turns=data['turns'].split(',')
				turns=[float(t) for t in self.turns]
					
				self.iters=int(data['iters'])
				self.count=0
				self.size=data['size'].split(',')
				print self.size
				formatArray=GeomVertexArrayFormat()
				formatArray.addColumn(InternalName.make("drawFlag"), 1, Geom.NTUint8, Geom.COther)
		
				format=GeomVertexFormat(GeomVertexFormat.getV3n3cpt2())
				format.addArray(formatArray)
				self.format=GeomVertexFormat.registerFormat(format)
		
				bodydata=GeomVertexData("body vertices", self.format, Geom.UHStatic)
				self.nPath=NodePath(self.nodeName)
				print self.nPath
				self.nPath.setShaderAuto()
				if self.bark_remote=='1':
					self.localbark='textures/trees/'+self.bark.replace('://','.').replace('/','.')
					lfile=Filename.binaryFilename(self.localbark)
					if lfile.exists():
						self.barkTexture=loader.loadTexture(self.localbark)
					else:
						self.barkTexture=loader.loadTexture(DEFAULTBARK)
						getbark=threading.Thread(name='GetBark'+str(data['x'])+'-'+str(data['y'])+'-'+str(data['z']),target=self.getBark)
						getbark.start()
				else:	
					self.barkTexture=loader.loadTexture(self.bark)
					
				if self.leaf_remote=='1':
					self.localleaf='textures/trees/'+self.leaf.replace('://','.').replace('/','.')
					lfile=Filename.binaryFilename(self.localleaf)
					if lfile.exists():
						self.leafTexture=loader.loadTexture(self.localleaf)
					else:
						self.leafTexture=loader.loadTexture(DEFAULTLEAF)
						getleaf=threading.Thread(name='GetLeaf'+str(data['x'])+'-'+str(data['y'])+'-'+str(data['z']),target=self.getLeaf)
						getleaf.start()
				else:
					self.leafTexture=loader.loadTexture(self.leaf)	
					
				self.nPath.setTexture(self.barkTexture)
				
				self.makeFractalTree(bodydata=bodydata,length=Vec3(float(self.size[0]),float(self.size[1]),float(self.size[2])),pos=Vec3(0,0,0),numIterations=self.iters)
			
				print 'bark',self.bark
				print 'barktext',self.barkTexture
				print self.nPath
				self.nPath.reparentTo(render.find('Objects/Trees'))
				self.nPath.setPos(float(data['x']),float(data['y']),float(data['z']))
				self.nPath.setH(self.heading)
				#self.nPath.setScale(1,1,self.scale)
				
			else:
				del(self)
		else:
			del(self)
	
	def getVarSpec(self):
		return {'x':'float','y':'float','z':'float','type':'str','size':'str',
			'bark':'str','bark_remote':'int','leaf':'str','leaf_remote':'int',
			'leafModel':'str','turns':'str','heading':'float','scale':'float'}	
	def makeFractalTree(self,bodydata, length=None, pos=Vec3(0,0,0), numIterations=11, numCopies=4,vecList=[Vec3(0,0,1),Vec3(1,0,0), Vec3(0,-1,0)]):
		#print length,pos,numIterations
		if numIterations>0:
	
			self.drawBody( bodydata, pos, vecList, length.getX())
			
			#move foward along the right axis
			
			newPos=pos+vecList[0]*length.length()
	
		
			#only branch every third level (sorta)
			if numIterations%3==0:
				#decrease dimensions when we branch			
				length=Vec3(length.getX()/2, length.getY()/2, length.getZ()/1.1)
				for i in range(numCopies):
					self.makeFractalTree(bodydata, length,newPos, numIterations-1, numCopies,self.randomAxis(vecList,numIterations-1))
			else:
				#just make another branch connected to this one with a small variation in direction
				self.makeFractalTree(bodydata, length,newPos, numIterations-1,numCopies,self.smallRandomAxis(vecList,numIterations-1))
	
		else:
			self.drawBody(bodydata, pos, vecList, length.getX(),False)
			self.drawLeaf(bodydata, pos,vecList)
			
	def drawLeaf(self,vdata,pos=Vec3(0,0,0),vecList=[Vec3(0,0,1), Vec3(1,0,0),Vec3(0,-1,0)], scale=0.125):
	
		#use the vectors that describe the direction the branch grows to make the right 
			#rotation matrix
		newCs=Mat4(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0)
		newCs.setRow(0, vecList[2]) #right
		newCs.setRow(1, vecList[1]) #up
		newCs.setRow(2, vecList[0]) #forward
		newCs.setRow(3, Vec3(0,0,0))                                  
		newCs.setCol(3,Vec4(0,0,0,1))
	
		axisAdj=Mat4.scaleMat(scale)*newCs*Mat4.translateMat(pos)	
		
		leafModel=loader.loadModelCopy(self.leafModel)
	
	
		leafModel.reparentTo(self.nPath)
		leafModel.setTexture(self.leafTexture)
		leafModel.setTransform(TransformState.makeMat(axisAdj))

	def drawBody(self, vdata, pos, vecList, radius=1, keepDrawing=True,numVertices=8):

		circleGeom=Geom(vdata)
	
		vertWriter=GeomVertexWriter(vdata, "vertex")
		colorWriter=GeomVertexWriter(vdata, "color")
		normalWriter=GeomVertexWriter(vdata, "normal")
		drawReWriter=GeomVertexRewriter(vdata, "drawFlag")
		texReWriter=GeomVertexRewriter(vdata, "texcoord")
		
		
		startRow=vdata.getNumRows()
		vertWriter.setRow(startRow)
		colorWriter.setRow(startRow)
		normalWriter.setRow(startRow)
		
		sCoord=0
	
		if (startRow!=0):
			texReWriter.setRow(startRow-numVertices)
			sCoord=texReWriter.getData2f().getX()+1
			
			drawReWriter.setRow(startRow-numVertices)
			if(drawReWriter.getData1f()==False):
				sCoord-=1
		
		drawReWriter.setRow(startRow)
		texReWriter.setRow(startRow)	
		
		angleSlice=2*math.pi/numVertices
		currAngle=0
				
		#axisAdj=Mat4.rotateMat(45, axis)*Mat4.scaleMat(radius)*Mat4.translateMat(pos)
	
		perp1=vecList[1]
		perp2=vecList[2]	
	
		#vertex information is written here
		for i in range(numVertices):
			adjCircle=pos+(perp1*math.cos(currAngle)+perp2*math.sin(currAngle))*radius
			normal=perp1*math.cos(currAngle)+perp2*math.sin(currAngle)		
			normalWriter.addData3f(normal)
			vertWriter.addData3f(adjCircle)
			texReWriter.addData2f(sCoord,(i+0.001)/(numVertices-1))
			colorWriter.addData4f(0.5,0.5,0.5,1)
			drawReWriter.addData1f(keepDrawing)
			currAngle+=angleSlice
	
		
		drawReader=GeomVertexReader(vdata, "drawFlag")
		drawReader.setRow(startRow-numVertices)
	
		#we cant draw quads directly so we use Tristrips
		if (startRow!=0) & (drawReader.getData1f()!=False):
			lines=GeomTristrips(Geom.UHStatic)
			half=int(numVertices*0.5)
			for i in range(numVertices):
				lines.addVertex(i+startRow)
				if i< half:
					lines.addVertex(i+startRow-half)
				else:
					lines.addVertex(i+startRow-half-numVertices)
	
			lines.addVertex(startRow)
			lines.addVertex(startRow-half)
			lines.closePrimitive()
			lines.decompose()
			circleGeom.addPrimitive(lines)
			
	
			circleGeomNode=GeomNode("Debug")
			circleGeomNode.addGeom(circleGeom)
	
			#I accidentally made the front-face face inwards. Make reverse makes the tree render properly and
			#should cause any surprises to any poor programmer that tries to use this code
			circleGeomNode.setAttrib(CullFaceAttrib.makeReverse(),1)
			global numPrimitives
			numPrimitives+=numVertices*2
		
			self.nPath.attachNewNode(circleGeomNode)	
	
	#this computes the new Axis which we'll make a branch grow alowng when we split
	def randomAxis(self,vecList,numIterations):
		fwd=vecList[0]*self.scale
		perp1=vecList[1]	
		perp2=vecList[2]
		self.count+=1.68
		#nfwd=fwd+perp1*(2*random.random()-1) + perp2*(2*random.random()-1)
		rand=(self.count % 14) /14
		#print "RANDAX:"+str(self.count)+":"+str(rand)
		duh=perp1*2.5
		nfwd=fwd+perp1*(2*float(self.turns[int(self.count % 32)])*rand-1) + perp2*(2*float(self.turns[int((self.count+5) % 21)])*rand-1)
		nfwd.normalize()
		                            
		nperp2=nfwd.cross(perp1)
		nperp2.normalize()
		
		nperp1=nfwd.cross(nperp2)
		nperp1.normalize()
	
		return [nfwd, nperp1, nperp2]
		
	
	#this makes smalle variations in direction when we are growing a branch but not splitting
	def smallRandomAxis(self,vecList,numIterations):		
		fwd=vecList[0]
		perp1=vecList[1]
		perp2=vecList[2]
		self.count+=.68
		rand=(self.count % 14) /14
		#print "SMRANDAX:"+str(self.count)+":"+str(rand)+":"+str(self.turns[int(self.count % 7)])
		
		#nfwd=fwd+perp1*(1*self.turns[2]-0.5) + perp2*(1*self.turns[3]-0.5)
		#nfwd=fwd+perp1*(1*self.count/self.iters-0.5) + perp2*(1*self.count/self.iters-0.5)
		#nfwd=fwd+perp1*(1*self.turns[rand]-0.5) + perp2*(1*self.turns[rand]-0.5)
		#print type(self.turns[int(self.count % 7)])
		#print perp1*self.turns[int(self.count % 7)]
		nfwd=fwd+perp1*(1*float(self.turns[int(self.count % 21)])*rand-0.5) + perp2*(1*float(self.turns[int((self.count+5) % 32)])*rand-0.5)
		if numIterations>(self.iters-3):
				nfwd=nfwd/10
		nfwd.normalize()
		
		nperp2=nfwd.cross(perp1)
		nperp2.normalize()
		
		nperp1=nfwd.cross(nperp2)
		nperp1.normalize()
	
		return [nfwd, nperp1, nperp2]
	
	# fired when files are downloaded
	def barkWatch(self,count=0,size=0,total=0):
		if count*size==total:
			print 'equaled'
			self.barkTexture=loader.loadTexture(self.localbark)
			self.nPath.setTexture(self.barkTexture,1)
		else:
			print count,size,total
			
	def getBark(self):
		(loadfilename,loadheaders)=urllib.urlretrieve(self.bark,self.localbark)	
		self.nPath.setTexture(loader.loadTexture(loadfilename),1)
		
	def getLeaf(self):
		(loadfilename,loadheaders)=urllib.urlretrieve(self.leaf,self.localleaf)	
		self.leafTexture=loader.loadTexture(loadfilename)
