__author__='sigilis.com'
__version__='0.2'
from pandac.PandaModules import Filename,InternalName
from pandac.PandaModules import Texture,TextureStage
from pandac.PandaModules import TexGenAttrib
from pandac.PandaModules import Vec3
from pandac.PandaModules import Geom, GeomNode, GeomTrifans, GeomTristrips
from pandac.PandaModules import GeomVertexReader, GeomVertexWriter
from pandac.PandaModules import GeomVertexRewriter, GeomVertexData
from pandac.PandaModules import GeomVertexArrayFormat, GeomVertexFormat
from pandac.PandaModules import NodePath,BitMask32

from SigilisModel import SigilisModel

class Wall(SigilisModel): 
	def __init__(self,data=None,user=None):
		if data !=None:
			SigilisModel.__init__(self,user=user,data=data,modeltype='Wall')
			if self.good==True:
				self.tileInner=data['tile-inner']
				self.tileOuter=data['tile-outer']
				self.tileIremote=int(data['tile_i_remote'])
				self.tileOremote=int(data['tile_o_remote'])
				self.tile_scale=float(data['tile_scale'])
				self.heading=float(data['heading'])	
				self.window=data['window'].split(',')	
				self.door=data['door'].split(',')
				#self.vertices=int(data['vertices'])
				
				formatArray=GeomVertexArrayFormat()
				formatArray.addColumn(InternalName.make("drawFlag"), 1, Geom.NTUint8, Geom.COther)
		
				format=GeomVertexFormat(GeomVertexFormat.getV3n3cpt2())
				format.addArray(formatArray)
				self.format=GeomVertexFormat.registerFormat(format)
		
				innerdata=GeomVertexData("inner vertices", self.format, Geom.UHStatic)
				outerdata=GeomVertexData("outer vertices", self.format, Geom.UHStatic)
				
				
				self.nPath.node().setIntoCollideMask(BitMask32.bit(19))
				self.inner=NodePath("inner")
				self.inner.node().setIntoCollideMask(BitMask32.bit(19))
				self.outer=NodePath("outer")			
				self.outer.node().setIntoCollideMask(BitMask32.bit(19))
				
				self.inner.reparentTo(self.nPath)		
				self.outer.reparentTo(self.nPath)
				
				ts = TextureStage('ts')
				ts.setMode(TextureStage.MDecal)
				
				innerGeom=self.makeWall(0.1,'inner',innerdata)
				outerGeom=self.makeWall(0,'outer',outerdata)
				
				inwallTexture=loader.loadTexture(self.tileInner)
				inwallTexture.setWrapU(Texture.WMClamp)
				inwallTexture.setWrapV(Texture.WMClamp)
				
				self.inner.setTexture(inwallTexture)
				self.inner.setTwoSided(False)
				self.inner.attachNewNode(innerGeom)
				
				outwallTexture=loader.loadTexture(self.tileOuter)
				outwallTexture.setWrapU(Texture.WMClamp)
				outwallTexture.setWrapV(Texture.WMClamp)
				
				self.outer.setTexture(outwallTexture)
				self.outer.setTwoSided(False)
				self.outer.attachNewNode(outerGeom)
				
				self.nPath.reparentTo(render.find('Objects/Walls'))	
				self.nPath.setPos(self.x,self.y,self.z)
				self.nPath.setH(self.heading)		
			else:
				del(self)

	def getVarSpec(self):
		return {'x':['float',0,'text','X position'],'y':['float',0,'text','Y position'],'z':['float',0,'text','Z position'],
			'heading':['float',1,'text','Heading'],'width':['float',2,'text','Wall width'],'height':['float',2,'text','Wall height'],
			'tile_inner':['str',3,'text/file','Inner wall texture local relative path or URL|or, Upload File'],
			'tile_i_remote':['bool',4,'radio','Is texture remote?'],
			'tile_outer':['str',3,'text/file','Outer wall texture local relative path or URL|or, Upload File'],
			'tile_o_remote':['bool',4,'radio','Is texture remote?'],'tile_scale':['float',5,'text','Tile Scale Factor'],
			'window':['str',7,'text','Window Dimensions'],
			'door':['str',7,'text','Door Dimensions'],
			'reqtosubmit':['str',0,'hidden','x,y,z,width,height,tile_inner,tile_outer'],
			'defaults':['str',0,'hidden','heading=0,tile_scale=1']}
	
			
	def makeWall(self, atY,inOut,vdata):			
		wallGeom=Geom(vdata)
	
		vertWriter=GeomVertexWriter(vdata, "vertex")
		normalWriter=GeomVertexWriter(vdata, "normal")
		colorWriter=GeomVertexWriter(vdata, "color")
		uvWriter=GeomVertexWriter(vdata, "texcoord")
		drawWriter=GeomVertexWriter(vdata, "drawFlag")
		
		vertWriter.setRow(0)
		colorWriter.setRow(0)
		uvWriter.setRow(0)
		normalWriter.setRow(0)
		drawWriter.setRow(0)
		
		
		for i in range(2):
			for j in range(2):
				if inOut=='inner':
					position=Vec3(0-(j*self.width)+self.width, atY,0+(i*self.height))
					#uvWriter.addData2f(1-j,1-i)
				else:
					position=Vec3(0+(j*self.width), atY,0+(i*self.height))
				vertWriter.addData3f(position)
				uvWriter.addData2f(j,i)	
				colorWriter.addData4f(1.0, 1.0, 1.0, 0.0)
				#position.setZ(position.getZ()*direction)
				position.normalize()
				normalWriter.addData3f(position)
		
		wall=GeomTrifans(Geom.UHStatic)
		if float(self.door[0])>0:
			doorpos0=Vec3(float(self.door[0])*self.width, atY,0)
			vertWriter.addData3f(doorpos0)
			uvWriter.addData2f(float(self.door[0]),0)	
			colorWriter.addData4f(1.0, 1.0, 1.0, 0.0)
			doorpos0.normalize()
			normalWriter.addData3f(doorpos0)
			
			doorpos1=Vec3(float(self.door[0])*self.width+float(self.door[1]), atY,0)
			vertWriter.addData3f(doorpos1)
			uvWriter.addData2f(float(self.door[0])+float(self.door[1])/self.width,0)	
			colorWriter.addData4f(1.0, 1.0, 1.0, 0.0)
			doorpos1.normalize()
			normalWriter.addData3f(doorpos1)
			
			doorpos2=Vec3(float(self.door[0])*self.width, atY,float(self.door[2]))
			vertWriter.addData3f(doorpos2)
			uvWriter.addData2f(float(self.door[0]),float(self.door[2])/self.height)	
			colorWriter.addData4f(1.0, 1.0, 1.0, 0.0)
			doorpos2.normalize()
			normalWriter.addData3f(doorpos2)
			
			doorpos3=Vec3(float(self.door[0])*self.width+float(self.door[1]), atY,float(self.door[2]))
			vertWriter.addData3f(doorpos3)
			uvWriter.addData2f(float(self.door[0])+float(self.door[1])/self.width,float(self.door[2])/self.height)	
			colorWriter.addData4f(1.0, 1.0, 1.0, 0.0)
			doorpos3.normalize()
			normalWriter.addData3f(doorpos3)
				
		if float(self.window[0])>0:
			winpos0=Vec3(float(self.window[0])*self.width, atY,float(self.window[1])*self.height)
			vertWriter.addData3f(winpos0)
			uvWriter.addData2f(float(self.window[0]),float(self.window[1]))	
			colorWriter.addData4f(1.0, 1.0, 1.0, 0.0)
			position.normalize()
			normalWriter.addData3f(winpos0)
			
			winpos1=Vec3(float(self.window[0])*self.width+float(self.window[2]), atY,float(self.window[1])*self.height)
			vertWriter.addData3f(winpos1)
			uvWriter.addData2f(float(self.window[0])+float(self.window[2])/self.width,float(self.window[1]))	
			colorWriter.addData4f(1.0, 1.0, 1.0, 0.0)
			position.normalize()
			normalWriter.addData3f(winpos1)
			
			winpos2=Vec3(float(self.window[0])*self.width, atY,float(self.window[1])*self.height+float(self.window[3]))
			vertWriter.addData3f(winpos2)
			uvWriter.addData2f(float(self.window[0]),float(self.window[1])+float(self.window[3])/self.height)	
			colorWriter.addData4f(1.0, 1.0, 1.0, 0.0)
			position.normalize()
			normalWriter.addData3f(winpos2)
			
			winpos3=Vec3(float(self.window[0])*self.width+float(self.window[2]), atY,float(self.window[1])*self.height+float(self.window[3]))
			vertWriter.addData3f(winpos3)
			uvWriter.addData2f(float(self.window[0])+float(self.window[2])/self.width,float(self.window[1])+float(self.window[3])/self.height)	
			colorWriter.addData4f(1.0, 1.0, 1.0, 0.0)
			position.normalize()
			normalWriter.addData3f(winpos3)

		if float(self.door[0])>0:
			if float(self.window[0])>0:
				if float(self.window[0])>float(self.door[0]):
					if inOut=='inner':
						wall.addVertex(1)		
						wall.addVertex(3)		
						wall.addVertex(6)		
						wall.addVertex(4)		
					else:
						wall.addVertex(0)		
						wall.addVertex(4)		
						wall.addVertex(6)		
						wall.addVertex(2)		
					wall.closePrimitive()	
					wallGeom.addPrimitive(wall)	
					
					wall=GeomTrifans(Geom.UHStatic)
					if inOut=='inner':
						wall.addVertex(3)
						wall.addVertex(2)		
						wall.addVertex(11)		
						wall.addVertex(10)		
					else:			
						wall.addVertex(3)
						wall.addVertex(2)		
						wall.addVertex(10)		
						wall.addVertex(11)	
					wall.closePrimitive()	
					wallGeom.addPrimitive(wall)	
					
					wall=GeomTrifans(Geom.UHStatic)
					if inOut=='inner':
						wall.addVertex(3)
						wall.addVertex(10)		
						wall.addVertex(7)		
						wall.addVertex(6)		
					else:			
						wall.addVertex(2)
						wall.addVertex(6)		
						wall.addVertex(7)		
						wall.addVertex(10)	
					wall.closePrimitive()	
					wallGeom.addPrimitive(wall)	
					
					wall=GeomTrifans(Geom.UHStatic)
					if inOut=='inner':
						wall.addVertex(5)
						wall.addVertex(7)		
						wall.addVertex(10)		
						wall.addVertex(8)		
					else:			
						wall.addVertex(5)
						wall.addVertex(8)		
						wall.addVertex(10)		
						wall.addVertex(7)	
					wall.closePrimitive()	
					wallGeom.addPrimitive(wall)	
					
					wall=GeomTrifans(Geom.UHStatic)
					if inOut=='inner':
						wall.addVertex(5)
						wall.addVertex(8)		
						wall.addVertex(9)		
						wall.addVertex(0)		
					else:			
						wall.addVertex(5)
						wall.addVertex(1)		
						wall.addVertex(9)		
						wall.addVertex(8)
					wall.closePrimitive()	
					wallGeom.addPrimitive(wall)	
					
					wall=GeomTrifans(Geom.UHStatic)
					if inOut=='inner':
						wall.addVertex(0)
						wall.addVertex(9)		
						wall.addVertex(11)		
						wall.addVertex(2)		
					else:			
						wall.addVertex(1)
						wall.addVertex(3)		
						wall.addVertex(11)		
						wall.addVertex(9)
						
				else:		# window left
					
					if inOut=='inner':
						wall.addVertex(1)		
						wall.addVertex(3)		
						wall.addVertex(10)		
						wall.addVertex(8)		
					else:
						wall.addVertex(0)		
						wall.addVertex(8)		
						wall.addVertex(10)		
						wall.addVertex(2)	
					wall.closePrimitive()	
					wallGeom.addPrimitive(wall)	
					
					wall=GeomTrifans(Geom.UHStatic)
					if inOut=='inner':
						wall.addVertex(3)
						wall.addVertex(2)		
						wall.addVertex(11)		
						wall.addVertex(10)		
					else:			
						wall.addVertex(3)
						wall.addVertex(2)		
						wall.addVertex(10)		
						wall.addVertex(11)	
					wall.closePrimitive()	
					wallGeom.addPrimitive(wall)	
					
					wall=GeomTrifans(Geom.UHStatic)
					if inOut=='inner':
						wall.addVertex(2)
						wall.addVertex(7)		
						wall.addVertex(6)		
						wall.addVertex(11)		
					else:			
						wall.addVertex(3)
						wall.addVertex(11)		
						wall.addVertex(6)		
						wall.addVertex(7)	
					wall.closePrimitive()	
					wallGeom.addPrimitive(wall)	
					
					wall=GeomTrifans(Geom.UHStatic)
					if inOut=='inner':
						wall.addVertex(4)
						wall.addVertex(9)		
						wall.addVertex(11)		
						wall.addVertex(6)		
					else:			
						wall.addVertex(4)
						wall.addVertex(6)		
						wall.addVertex(11)		
						wall.addVertex(9)	
					wall.closePrimitive()	
					wallGeom.addPrimitive(wall)	
					
					wall=GeomTrifans(Geom.UHStatic)
					if inOut=='inner':
						wall.addVertex(4)
						wall.addVertex(1)		
						wall.addVertex(8)		
						wall.addVertex(9)		
					else:			
						wall.addVertex(0)
						wall.addVertex(4)		
						wall.addVertex(9)		
						wall.addVertex(8)		
					wall.closePrimitive()	
					wallGeom.addPrimitive(wall)	
					
					wall=GeomTrifans(Geom.UHStatic)
					if inOut=='inner':
						wall.addVertex(0)
						wall.addVertex(5)		
						wall.addVertex(7)		
						wall.addVertex(2)		
					else:			
						wall.addVertex(5)
						wall.addVertex(1)		
						wall.addVertex(3)		
						wall.addVertex(7)				
			else:	# just door
				if inOut=='inner':
					wall.addVertex(0)		
					wall.addVertex(5)		
					wall.addVertex(7)		
					wall.addVertex(2)		
				else:
					wall.addVertex(0)		
					wall.addVertex(4)		
					wall.addVertex(6)		
					wall.addVertex(2)			
				wall.closePrimitive()	
				wallGeom.addPrimitive(wall)	
				
				wall=GeomTrifans(Geom.UHStatic)
				if inOut=='inner':
					wall.addVertex(1)		
					wall.addVertex(3)		
					wall.addVertex(6)		
					wall.addVertex(4)		
				else:	
					wall.addVertex(1)		
					wall.addVertex(3)		
					wall.addVertex(7)		
					wall.addVertex(5)
				wall.closePrimitive()	
				wallGeom.addPrimitive(wall)	
				
				wall=GeomTrifans(Geom.UHStatic)
				if inOut=='inner':
					wall.addVertex(3)		
					wall.addVertex(2)		
					wall.addVertex(7)		
					wall.addVertex(6)		
				else:	
					wall.addVertex(3)		
					wall.addVertex(2)		
					wall.addVertex(6)		
					wall.addVertex(7)	
					
		else:	
			if float(self.window[0])>0:
				if inOut=='inner':
					wall.addVertex(0)		
					wall.addVertex(5)		
					wall.addVertex(7)		
					wall.addVertex(2)		
				else:
					wall.addVertex(0)		
					wall.addVertex(4)		
					wall.addVertex(6)		
					wall.addVertex(2)			
				wall.closePrimitive()	
				wallGeom.addPrimitive(wall)	
				
				wall=GeomTrifans(Geom.UHStatic)
				if inOut=='inner':
					wall.addVertex(0)		
					wall.addVertex(1)		
					wall.addVertex(4)		
					wall.addVertex(5)		
				else:	
					wall.addVertex(0)		
					wall.addVertex(1)		
					wall.addVertex(5)		
					wall.addVertex(4)		
				wall.closePrimitive()	
				wallGeom.addPrimitive(wall)	
				
				wall=GeomTrifans(Geom.UHStatic)
				if inOut=='inner':
					wall.addVertex(1)		
					wall.addVertex(3)		
					wall.addVertex(6)		
					wall.addVertex(4)		
				else:	
					wall.addVertex(1)		
					wall.addVertex(3)		
					wall.addVertex(7)		
					wall.addVertex(5)
				wall.closePrimitive()	
				wallGeom.addPrimitive(wall)	
				
				wall=GeomTrifans(Geom.UHStatic)
				if inOut=='inner':
					wall.addVertex(3)		
					wall.addVertex(2)		
					wall.addVertex(7)		
					wall.addVertex(6)		
				else:	
					wall.addVertex(3)		
					wall.addVertex(2)		
					wall.addVertex(6)		
					wall.addVertex(7)	
				
				
			else:
				wall.addVertex(0)		
				wall.addVertex(1)
				wall.addVertex(3)
				wall.addVertex(2)
			
		wall.closePrimitive()
	
		wallGeom.addPrimitive(wall)
		
		
		wallGeom.decomposeInPlace()
		## maybe later???
		wallGeom.unifyInPlace(4,False)
		
		wallGeomNode=GeomNode("WallGeom:"+inOut)
		wallGeomNode.addGeom(wallGeom)
			
		return wallGeomNode
